﻿namespace TabsStudioXMenu
{
    public class XMenu : TabsStudioExt.ITabsStudioAddin
    {
        public XMenu()
        {
            defaultItems = new System.Collections.Generic.List<string>();
            defaultItems.Add("File.SaveSelectedItems");
            defaultItems.Add("Window.CloseDocumentWindow");
            defaultItems.Add("File.CloseAllButThis");
            defaultItems.Add("File.CopyFullPath");
            defaultItems.Add("File.OpenContainingFolder");
            defaultItems.Add("Window.Floating");
            defaultItems.Add("Window.Float");
            defaultItems.Add("Window.Dockable");
            defaultItems.Add("Window.Dock");
            defaultItems.Add("Window.TabbedDocument");
            defaultItems.Add("Window.DockasTabbedDocument");
            defaultItems.Add("Window.AutoHide");
            defaultItems.Add("Window.Hide");
            defaultItems.Add("OtherContextMenus.EasyMDIDocumentWindow.Float");
            defaultItems.Add("OtherContextMenus.EasyMDIDocumentWindow.DockasTabbedDocument");
            defaultItems.Add("Window.TileHorizontally");
            defaultItems.Add("Window.NewHorizontalTabGroup");
            defaultItems.Add("Window.TileVertically");
            defaultItems.Add("Window.NewVerticalTabGroup");
            defaultItems.Add("Window.MovetoNextTabGroup");
            defaultItems.Add("Window.MovetoPreviousTabGroup");
            defaultItems.Add("OtherContextMenus.EasyMDIToolWindow.TfsAnnotateCopyFullPath");
        }

        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_)
        {
            engine = engine_;
            dte = dte_;
            commandInfo = new CommandInfo(dte);
            engine.OpeningTabExtensionContextMenu += OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu += OpeningTabContextMenu;
        }

        public void OnDisconnection()
        {
            engine.OpeningTabExtensionContextMenu -= OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu -= OpeningTabContextMenu;
        }

        private void OpeningTabExtensionContextMenu(object sender,
            TabsStudioExt.OpeningTabExtensionContextMenuEventArgs e)
        {
            if (e.Tabs.IsGroupSelected && e.Tab.IsTabSelected && e.TabExtension.IsExtensionActive)
                AddExternalCommands(e.ContextMenu, IsDocumentWindow(e.TabExtension.Window));
        }

        private void OpeningTabContextMenu(object sender, TabsStudioExt.OpeningTabContextMenuEventArgs e)
        {
            if (e.Tabs.IsGroupSelected && e.Tab.IsTabSelected)
            {
                foreach (TabsStudioExt.TabExtension ext in e.Tab.TabExtensions)
                {
                    if (ext.IsExtensionActive)
                    {
                        AddExternalCommands(e.ContextMenu, IsDocumentWindow(ext.Window));
                        break;
                    }
                }
            }
        }

        private void AddExternalCommands(System.Windows.Controls.ContextMenu menu, bool document)
        {
            string commandBarName = document ? "Easy MDI Document Window" : "Easy MDI Tool Window";
            Microsoft.VisualStudio.CommandBars.CommandBar objCommandBar =
                ((Microsoft.VisualStudio.CommandBars.CommandBars)dte.CommandBars)[commandBarName];
            if (objCommandBar != null)
            {
                System.Collections.ArrayList items = CreateExternalMenuItems(objCommandBar.Controls);
                if (items.Count > 0)
                    PlaceMenuItem(new System.Windows.Controls.Separator(), menu);
                foreach (object o in items)
                    PlaceMenuItem(o, menu);
            }
        }

        private System.Collections.ArrayList CreateExternalMenuItems(
            Microsoft.VisualStudio.CommandBars.CommandBarControls controls)
        {
            System.Collections.ArrayList result = new System.Collections.ArrayList();
            for (int i = 1; i <= controls.Count; ++i)
            {
                Microsoft.VisualStudio.CommandBars.CommandBarControl control = controls[i];
                CommandStatus commandStatus = commandInfo.GetCommandStatus(control);
                if (!commandStatus.IsVisible || IsDefaultCommand(control))
                    continue;
                
                if (control is Microsoft.VisualStudio.CommandBars.CommandBarButton ||
                    control is Microsoft.VisualStudio.CommandBars.CommandBarPopup)
                {
                    System.Windows.Controls.MenuItem menuItem = new System.Windows.Controls.MenuItem();
                    menuItem.Header = FormatMenuItemCaption(control.Caption);
                    menuItem.IsEnabled = commandStatus.IsEnabled;
                    menuItem.IsChecked = commandStatus.IsChecked;
                    if (control is Microsoft.VisualStudio.CommandBars.CommandBarButton)
                    {
                        Microsoft.VisualStudio.CommandBars.CommandBarButton button = control as
                            Microsoft.VisualStudio.CommandBars.CommandBarButton;
                        if (dte.Version.Equals("10.0"))
                        {
                            stdole.StdPicture picture = null;
                            try { picture = button.Picture; }
                            catch (System.Exception) { }
                            menuItem.Icon = StdOLEHelper.PictureToImage(picture);
                        }
                        else
                        {
                            menuItem.Icon = StdOLEHelper.PictureToImage(button.Picture, button.Mask);
                        }
                        menuItem.Click += delegate(object sender, System.Windows.RoutedEventArgs e) { control.Execute(); };
                    }
                    if (control is Microsoft.VisualStudio.CommandBars.CommandBarPopup)
                    {
                        System.Collections.ArrayList items = CreateExternalMenuItems(
                            (control as Microsoft.VisualStudio.CommandBars.CommandBarPopup).Controls);
                        if (items.Count == 0)
                            continue;
                        foreach (object o in items)
                            menuItem.Items.Add(o);
                    }
                    if (result.Count != 0 && control.BeginGroup)
                        result.Add(new System.Windows.Controls.Separator());
                    result.Add(menuItem);
                }
            }
            return result;
        }

        private void PlaceMenuItem(object menuItem, System.Windows.Controls.ContextMenu menu)
        {
            foreach (object item in menu.Items)
            {
                if (item is System.Windows.Controls.Control)
                {
                    if ((item as System.Windows.Controls.Control).Name.Equals("TabsStudioSeparator"))
                    {
                        menu.Items.Insert(menu.Items.IndexOf(item), menuItem);
                        break;
                    }
                }
            }
        }

        private string FormatMenuItemCaption(string s)
        {
            return s.Replace("&", "").Replace("_", "__");
        }

        private bool IsDefaultCommand(Microsoft.VisualStudio.CommandBars.CommandBarControl c)
        {
            return defaultItems.Contains(commandInfo.GetCommandName(c));
        }

        private bool IsDocumentWindow(EnvDTE.Window w)
        {
            return w.Type == EnvDTE.vsWindowType.vsWindowTypeDocument;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private EnvDTE80.DTE2 dte;
        private System.Collections.Generic.List<string> defaultItems;
        CommandInfo commandInfo;
    }
}